@@ -36,6 +36,7 @@ from xpander_sdk.modules.agents.models.agent import (
     AgentStatus,
     AgentType,
     DatabaseConnectionString,
+    LLMCredentials,
 )
 from xpander_sdk.modules.agents.models.knowledge_bases import AgentKnowledgeBase
 from xpander_sdk.modules.knowledge_bases.knowledge_bases_module import KnowledgeBases
@@ -90,6 +91,26 @@ class AgentGraph(XPanderSharedModel):
             if getattr(gi, attr, None) == value:
                 return gi
         return None
+    
+    @computed_field
+    @property
+    def sub_agents(self) -> List[str]:
+        """
+        Retrieve the list of sub-agent IDs associated with this agent.
+
+        This property returns the IDs of all agents that are nested under the current agent.
+        A sub-agent is identified when the `type` of the graph item matches `AgentGraphItemType.AGENT`.
+
+        Returns:
+            List[str]:  
+                A list of unique string IDs representing the agents nested under this agent.
+        """
+        return [
+            gi.item_id
+            for gi in self.items
+            if gi.type == AgentGraphItemType.AGENT
+        ]
+
 
 
 T = TypeVar("T", bound="Agent")
@@ -131,7 +152,7 @@ class Agent(XPanderSharedModel):
             type: Optional[AgentType]
             output_format: Optional[OutputFormat]
             output_schema: Optional[Dict]
-            llm_credentials_key: Optional[str]
+            llm_credentials: Optional[LLMCredentials]
             expected_output: Optional[str]
             agno_settings: Optional[AgnoSettings]
 
@@ -171,7 +192,7 @@ class Agent(XPanderSharedModel):
     output_format: Optional[OutputFormat] = OutputFormat.Markdown
     output_schema: Optional[Dict] = None
 
-    llm_credentials_key: Optional[str] = None
+    llm_credentials: Optional[LLMCredentials] = None
     expected_output: Optional[str] = ""
     agno_settings: Optional[AgnoSettings] = AgnoSettings()
 
@@ -574,10 +595,11 @@ class Agent(XPanderSharedModel):
         schema = get_db_schema_name(agent_id=self.id)
 
         return PostgresStorage(
-            table_name="agent_sessions",
+            table_name="team_sessions" if self.agno_settings.coordinate_mode else "agent_sessions",
             schema=schema,
             db_url=connection_string.connection_uri.uri,
             auto_upgrade_schema=True,
+            mode="team" if self.agno_settings.coordinate_mode else "agent"
         )
 
     def get_storage(self) -> Any:
