@@ -5,36 +5,68 @@ from typing import Any, Dict, List, Optional
 
 from loguru import logger
 
+from xpander_sdk import Configuration
 from xpander_sdk.models.shared import OutputFormat
+from xpander_sdk.modules.agents.agents_module import Agents
+from xpander_sdk.modules.agents.models.agent import AgentGraphItemType
 from xpander_sdk.modules.agents.sub_modules.agent import Agent
 from xpander_sdk.modules.tasks.sub_modules.task import Task
-from xpander_sdk.modules.tools_repository.models.mcp import MCPServerType
+from xpander_sdk.modules.tools_repository.models.mcp import MCPServerTransport, MCPServerType
 from xpander_sdk.modules.tools_repository.utils.schemas import build_model_from_schema
+from agno.agent import Agent as AgnoAgent
 
 async def build_agent_args(
     xpander_agent: Agent,
     task: Optional[Task] = None,
     override: Optional[Dict[str, Any]] = None
 ) -> Dict[str, Any]:
-    model = _load_llm_model(xpander_agent, override)
+    model = _load_llm_model(agent=xpander_agent, override=override)
     args: Dict[str, Any] = {}
 
-    _configure_output(args, xpander_agent, task)
-    _configure_session_storage(args, xpander_agent, task)
-    _configure_user_memory(args, xpander_agent, task)
-    await _attach_async_dependencies(args, xpander_agent, task, model)
-    _configure_knowledge_bases(args, xpander_agent)
-    _configure_additional_context(args, xpander_agent, task)
+    _configure_output(args=args, agent=xpander_agent, task=task)
+    _configure_session_storage(args=args, agent=xpander_agent, task=task)
+    _configure_user_memory(args=args, agent=xpander_agent, task=task)
+    await _attach_async_dependencies(args=args, agent=xpander_agent, task=task, model=model)
+    _configure_knowledge_bases(args=args, agent=xpander_agent)
+    _configure_additional_context(args=args, agent=xpander_agent, task=task)
 
-    args["tools"] = await _resolve_agent_tools(xpander_agent)
+    args["tools"] = await _resolve_agent_tools(agent=xpander_agent)
 
+    # team
+    if xpander_agent.agno_settings.coordinate_mode:
+        sub_agents = xpander_agent.graph.sub_agents
+        
+        # load sub agents
+        sub_agents = await asyncio.gather(
+            *[Agents(configuration=Configuration(api_key=xpander_agent.configuration.api_key,organization_id=xpander_agent.configuration.organization_id,base_url=xpander_agent.configuration.base_url)).aget(agent_id=sub_agent.item_id) for sub_agent in sub_agents]
+        )
+        # convert to members
+        members = await asyncio.gather(
+            *[build_agent_args(xpander_agent=sub_agent,override=override) for sub_agent in sub_agents]
+        )
+        
+        args.update({
+            "team_id": xpander_agent.id,
+            "success_criteria": xpander_agent.instructions.goal_str,
+            "mode": "coordinate",
+            "members": [AgnoAgent(**member) for member in members],
+            "add_member_tools_to_system_message": True,
+            "enable_agentic_context": True,
+            "enable_team_history": True,
+            "share_member_interactions": True,
+            "show_members_responses": True
+        })
+    else: # regular agent
+        args.update({
+            "agent_id": xpander_agent.id,
+            "goal": xpander_agent.instructions.goal_str,
+        })
+    
     args.update({
-        "agent_id": xpander_agent.id,
         "name": xpander_agent.name,
         "model": model,
         "description": xpander_agent.instructions.description,
         "instructions": xpander_agent.instructions.instructions,
-        "goal": xpander_agent.instructions.goal_str,
         "expected_output": task.expected_output if task and task.expected_output else xpander_agent.expected_output,
         "show_tool_calls": True,
         "add_datetime_to_instructions": True,
@@ -52,13 +84,33 @@ def _load_llm_model(agent: Agent, override: Optional[Dict[str, Any]]) -> Any:
 
     provider = agent.model_provider.lower()
 
+    is_xpander_cloud = getenv("IS_XPANDER_CLOUD", "false") == "true"
+    has_custom_llm_key = True if agent.llm_credentials and agent.llm_credentials.value else False
+    
+    # if xpander cloud: llm key >> env
+    # if local: env >> llm key
+    def get_llm_key(env_var_name: str):
+        env_llm_key = getenv(env_var_name)
+        
+        # default
+        if not has_custom_llm_key:
+            return env_llm_key
+        
+        # xpander cloud: custom and fallback to env
+        if is_xpander_cloud:
+            return agent.llm_credentials.value or env_llm_key
+        else:
+            return env_llm_key or agent.llm_credentials.value
+    
+    
+    
     if provider == "openai":
         from agno.models.openai import OpenAIChat
-        return OpenAIChat(id=agent.model_name, api_key=getenv("AGENTS_OPENAI_API_KEY"))
+        return OpenAIChat(id=agent.model_name, api_key=get_llm_key("AGENTS_OPENAI_API_KEY"), temperature=0.0)
 
     elif provider == "anthropic":
         from agno.models.anthropic import Claude
-        return Claude(id=agent.model_name, api_key=getenv("ANTHROPIC_API_KEY"))
+        return Claude(id=agent.model_name, api_key=get_llm_key("ANTHROPIC_API_KEY"),temperature=0.0)
 
     raise NotImplementedError(f"Provider '{provider}' is not supported for agno agents.")
 
@@ -154,25 +206,22 @@ async def _resolve_agent_tools(agent: Agent) -> List[Any]:
     )
 
     mcp_tools: List[MCPTools] = []
-    has_aws_mcp = False
     is_xpander_cloud = getenv("IS_XPANDER_CLOUD", "false") == "true"
 
     for mcp in agent.mcp_servers:
+        transport = mcp.transport.value.lower()
         if mcp.type == MCPServerType.Local:
             
-            # protection for serverless xpander, load in readonly
-            has_aws_mcp = True if mcp.command and "awslabs.aws_api_mcp_server.server" in mcp.command else False
-            if has_aws_mcp and is_xpander_cloud:
-                if not mcp.env_vars or "AWS_ACCESS_KEY_ID" not in mcp.env_vars or "AWS_SECRET_ACCESS_KEY" not in mcp.env_vars:
-                    logger.warning(f"setting aws mcp to readonly to missing credentials - {mcp.model_dump_json()}")
-                    if not mcp.env_vars:
-                        mcp.env_vars = {}
-                    mcp.env_vars["READ_OPERATIONS_ONLY"] = "true"
+            # protection for serverless xpander
+            is_aws_mcp = True if mcp.command and "awslabs" in mcp.command else False
+            if is_aws_mcp and is_xpander_cloud:
+                logger.warning(f"skipping aws mcp on agent {agent.id} due to xpander serverless")
+                continue
             
             command_parts = shlex.split(mcp.command)
-            
             mcp_tools.append(
                 MCPTools(
+                    transport=transport,
                     server_params=StdioServerParameters(
                         command=command_parts[0],
                         args=command_parts[1:],
@@ -183,9 +232,7 @@ async def _resolve_agent_tools(agent: Agent) -> List[Any]:
                 )
             )
         elif mcp.url:
-            is_sse = ("mcp" in mcp.url and "xpander.ai" in mcp.url) or mcp.url.endswith("/sse")
-            transport = "sse" if is_sse else "streamable-http"
-            params_cls = SSEClientParams if is_sse else StreamableHTTPClientParams
+            params_cls = SSEClientParams if mcp.transport == MCPServerTransport.SSE else StreamableHTTPClientParams
             mcp_tools.append(
                 MCPTools(
                     transport=transport,
@@ -194,11 +241,4 @@ async def _resolve_agent_tools(agent: Agent) -> List[Any]:
                 )
             )
 
-    functions = agent.tools.functions + await asyncio.gather(*[mcp.__aenter__() for mcp in mcp_tools])
-    
-    # wait until the aws mcp is ready
-    if has_aws_mcp and is_xpander_cloud:
-        logger.info("waiting for 5s until aws mcp is ready")
-        await asyncio.sleep(5)
-    
-    return functions
+    return agent.tools.functions + await asyncio.gather(*[mcp.__aenter__() for mcp in mcp_tools])
