@@ -13,11 +13,17 @@ class MCPServerAuthType(str, Enum):
     CustomHeaders = "custom_headers"
     _None = "none"
 
+class MCPServerTransport(str, Enum):
+    STDIO = "stdio"
+    SSE = "sse"
+    HTTP_Transport = "streamable-http"
+
 class MCPServerDetails(BaseModel):
     type: Optional[MCPServerType] = MCPServerType.Remote
     name: Optional[str] = None
     command: Optional[str] = None
     url: Optional[str] = None
+    transport: Optional[MCPServerTransport] = MCPServerTransport.HTTP_Transport
     auth_type: Optional[MCPServerAuthType] = MCPServerAuthType._None
     api_key: Optional[str] = None
     headers: Optional[Dict] = {}
